# frozen_string_literal: true

require_relative 'core/object'
require_relative 'core/array'
require_relative 'core/file'
require_relative 'core/hash'
require_relative 'core/numeric'
require_relative 'core/string'
require_relative 'core/regexp'
require_relative 'core/quantifiable_stdout'
