# frozen_string_literal: true

require_relative 'ext/core'
require_relative 'ext/asciidoctor'
require_relative 'ext/pdf-core'
require_relative 'ext/prawn'
require_relative 'ext/prawn-svg'
require_relative 'ext/prawn-table'
